Closed

Struct Closed 

pub struct Closed<T>(pub T);
Restrict points to the inside of a shape.

Closed is a newtype that wraps a shape. It prevents bodies and sites from existing outside the shape. Bodies and sites are never wrapped, and there are no ghost sites.

Tuple Fields

0: T