Periodic

Struct Periodic 

pub struct Periodic<T> { /* private fields */ }
Describe a simulation space that repeats in one or more directions.

Periodic is a newtype that wraps a shape. Use it to set the boundary for a Microstate.

When bodies or sites exit the shape through one of the periodic sides of the shape, they are wrapped to the other side. Similarly, sites that are within the interaction range of one of the periodic sides appear as ghost sites just outside the opposite side. Depending on the shape type T, Periodic<T> might implement fully periodic boundaries or ones that are periodic in some directions and closed in others.

Periodic is implemented for the following shapes:

Example

use hoomd_geometry::shape::Rectangle;
use hoomd_microstate::boundary::Periodic;

let periodic =
    Periodic::new(2.5, Rectangle::with_equal_edges(10.0.try_into()?))?;

Implementations

Source

impl<T> Periodic<T>

pub fn new(maximum_interaction_range: f64, shape: T) -> Result<Self, Error>

Construct a new periodic boundary condition.

Errors

Error::InteractionRangeTooLarge when maximum_interaction_range is larger than the maximum allowable interaction range by the shape.

Example
use hoomd_geometry::shape::Rectangle;
use hoomd_microstate::boundary::Periodic;

let periodic =
    Periodic::new(2.5, Rectangle::with_equal_edges(10.0.try_into()?))?;
Source

impl<T> Periodic<T>

pub fn shape(&self) -> &T

Access the boundary’s shape.

Example
use hoomd_geometry::shape::Rectangle;
use hoomd_microstate::boundary::Periodic;

let periodic =
    Periodic::new(2.5, Rectangle::with_equal_edges(10.0.try_into()?))?;

assert_eq!(periodic.shape().edge_lengths[0].get(), 10.0);
pub fn maximum_interaction_range(&self) -> f64

Access the boundary’s maximum interaction range.

Example
use hoomd_geometry::shape::Rectangle;
use hoomd_microstate::boundary::Periodic;

let periodic =
    Periodic::new(2.5, Rectangle::with_equal_edges(10.0.try_into()?))?;

assert_eq!(periodic.maximum_interaction_range(), 2.5);