DynamicPoint

Struct DynamicPoint 

pub struct DynamicPoint<V>
where V: Outer,
{ pub position: V, pub mass: f64, pub momentum: V, pub net_force: V, pub net_virial: V::Tensor, }
A position in space with the properties necessary for translational motion in MD.

Use DynamicPoint as a Body property type.

A default DynamicPoint has a mass of 1.0. Position, momentum, and net force of $\vec{0}$, and a zero-tensor for net virial.

Example

use hoomd_microstate::property::DynamicPoint;
use hoomd_vector::Cartesian;

let dynamic_point = DynamicPoint {
    position: Cartesian::from([1.0, -3.0]),
    mass: 1.0,
    ..Default::default()
};

Fields

position: V

The location of the extended body in space $[\mathrm{length}]$.

mass: f64

The mass of the extended body $[\mathrm{mass}]$.

momentum: V

The translational momentum of the extended body $[\mathrm{mass} \cdot \mathrm{length}] \cdot \mathrm{time}^{-1}]$.

net_force: V

The net force applied to the body in a Microstate $[\mathrm{mass} \cdot \mathrm{length}] \cdot \mathrm{time}^{-2}]$.

net_virial: V::Tensor

The net virial applied to the body in a Microstate $[\mathrm{energy}]$.