MomentOfInertia

Trait MomentOfInertia 

pub trait MomentOfInertia {
    type MomentOfInertia;

    // Required methods
    fn moment_of_inertia(&self) -> &Self::MomentOfInertia;
    fn moment_of_inertia_mut(&mut self) -> &mut Self::MomentOfInertia;
}
A body’s resistance to a change in rotational motion: $I$

MomentOfInertia connects a body’s angular momentum to its angular velocity: $\vec{L} = I \vec{\omega}$.

hoomd_md does not compute or utilize the moment of inertia of sites.

Units

The units of MomentOfInertia are $[\mathrm{mass} \cdot \mathrm{length}^2]$.

Required Associated Types

type MomentOfInertia

Type that expresses the moment of inertia.

Required Methods

fn moment_of_inertia(&self) -> &Self::MomentOfInertia

The moment of inertia of this body $[\mathrm{mass} \cdot \mathrm{length}^2]$.

fn moment_of_inertia_mut(&mut self) -> &mut Self::MomentOfInertia

The mutable moment of inertia of this body $[\mathrm{mass} \cdot \mathrm{length}^2]$.

Implementors