Momentum

Trait Momentum 

pub trait Momentum {
    type Momentum;

    // Required methods
    fn momentum(&self) -> &Self::Momentum;
    fn momentum_mut(&mut self) -> &mut Self::Momentum;
    fn velocity(&self) -> Self::Momentum;
    fn set_velocity(&mut self, velocity: Self::Momentum);
}
The translational motion of a body: $\vec{p}$

Momentum describes the translational motion of the body relative to the origin of the system coordinate system.

hoomd_md does not compute or utilize the momentum of sites.

Units

Momentum vectors have units of $[ \mathrm{energy}^{1/2} \cdot \mathrm{mass}^{1/2}]$.

Required Associated Types

type Momentum

Type that can express momentum and velocity.

Required Methods

fn momentum(&self) -> &Self::Momentum

The momentum of this body $[ \mathrm{energy}^{1/2} \cdot \mathrm{mass}^{1/2}]$.

fn momentum_mut(&mut self) -> &mut Self::Momentum

The mutable momentum of this body $[ \mathrm{energy}^{1/2} \cdot \mathrm{mass}^{1/2}]$.

fn velocity(&self) -> Self::Momentum

The velocity of this body $[ \mathrm{energy}^{1/2} \cdot \mathrm{mass}^{-1/2}]$.

fn set_velocity(&mut self, velocity: Self::Momentum)

Change the velocity of this body.

Implementors

Source

impl<V> Momentum for DynamicPoint<V>
where V: Mul<f64, Output = V> + Div<f64, Output = V> + Copy + Outer,

Source

impl<V, R> Momentum for DynamicOrientedPoint<V, R>
where V: Mul<f64, Output = V> + Div<f64, Output = V> + Copy + Wedge + Outer, R: RotationalMotionTypes,