NetTorque

Trait NetTorque 

pub trait NetTorque {
    type NetTorque;

    // Required methods
    fn net_torque(&self) -> &Self::NetTorque;
    fn net_torque_mut(&mut self) -> &mut Self::NetTorque;
}
The total torque acting on a body: $\vec{\tau}$

NetTorque is set only for bodies that belong to a microstate. It is always in the system frame.

hoomd_md does not store the net force acting on individual sites. Use methods in hoomd_interaction to compute torques on sites when needed.

Units

The units of NetTorque are $[\mathrm{energy}]$.

Required Associated Types

type NetTorque

Type that can express the net torque on a site or body.

Required Methods

fn net_torque(&self) -> &Self::NetTorque

The net torque on this site or body $[\mathrm{energy}]$.

fn net_torque_mut(&mut self) -> &mut Self::NetTorque

The mutable net torque on this site or body $[\mathrm{energy}]$.

Implementors