RotationalMotionTypes

Trait RotationalMotionTypes 

pub trait RotationalMotionTypes {
    type MomentOfInertia;
    type AngularMomentum;
}
Moment of inertia and angular momentum types.

RotationalMotionTypes sets which structs store the moment of inertia and angular momentum for a given Rotation representation.

Required Associated Types

type MomentOfInertia

Type that stores the moment of inertia in the natural coordinate frame of the body’s local rotation.

type AngularMomentum

Type that stores the angular momentum.

Implementations on Foreign Types

Source

impl RotationalMotionTypes for Angle

Source

impl RotationalMotionTypes for Versor

Implementors