Transform

Trait Transform 

pub trait Transform<S> {
    // Required method
    fn transform(&self, site_properties: &S) -> S;
}
Take Site properties in the body frame into the system frame.

See the property module-level documentation for an example that implements Transform for a custom type.

Required Methods

fn transform(&self, site_properties: &S) -> S

Transform site properties.

Given site_properties in the body frame, transform returns the corresponding site properties in the system frame relative to the body properties in &self.

Implementors

Source

impl Transform<OrientedHyperbolicPoint<3, Angle>> for OrientedHyperbolicPoint<3, Angle>

Source

impl Transform<Point<Hyperbolic<3>>> for OrientedHyperbolicPoint<3, Angle>

Treat Point<Hyperbolic<3>> sites as constituents of oriented rigid bodies.

Source

impl Transform<Point<Hyperbolic<3>>> for Point<Hyperbolic<3>>

Source

impl Transform<Point<Hyperbolic<4>>> for Point<Hyperbolic<4>>

Source

impl Transform<Point<Spherical<3>>> for Point<Spherical<3>>

Source

impl Transform<Point<Spherical<4>>> for Point<Spherical<4>>

Source

impl<V, R> Transform<OrientedPoint<V, R>> for OrientedPoint<V, R>
where V: Vector, R: Rotate<V> + Rotation,

Treat OrientedPoint sites as constituents of oriented rigid bodies.

Source

impl<V, R> Transform<Point<V>> for OrientedPoint<V, R>
where V: Vector, R: Rotate<V>,

Treat Point sites as constituents of oriented rigid bodies.

Source

impl<const N: usize> Transform<Point<Cartesian<N>>> for Point<Cartesian<N>>

Move Point properties from the local body frame to the system frame.