pub struct Open;Expand description
Allow bodies and sites to exist anywhere in space.
Every point lies inside Open boundary conditions, bodies and sites
are never wrapped, and there are no ghost sites.