pub struct DynamicOrientedPoint<V, R>{
pub position: V,
pub orientation: R,
pub mass: f64,
pub momentum: V,
pub net_force: V,
pub net_virial: V::Tensor,
pub moment_of_inertia: R::MomentOfInertia,
pub angular_momentum: R::AngularMomentum,
pub net_torque: V::Bivector,
}Expand description
A position in space with the properties necessary for translational and rotational motion in MD.
Use DynamicOrientedPoint as a Body property type.
A default DynamicOrientedPoint has a mass of 1.0 and position, momentum,
and net force of $\vec{0}$, and a zero-tensor for net virial.
Orientation defaults to the identity. DynamicOrientedPoint<_, Angle> has a
default moment of inertia of 1.0. DynamicOrientedPoint<_, Versor> has a
default moment of inertia of [1.0, 1.0, 1.0].
Example
use hoomd_microstate::property::DynamicOrientedPoint;
use hoomd_vector::{Angle, Cartesian};
use std::f64::consts::PI;
let oriented_dynamic_point = DynamicOrientedPoint {
position: Cartesian::from([1.0, -3.0]),
orientation: Angle::from(PI / 4.0),
..Default::default()
};Fields
position: VThe location of the extended body in space $[\mathrm{length}]$.
orientation: RRotate from the body’s reference frame to the system frame.
mass: f64The mass of the extended body $[\mathrm{mass}]$.
momentum: VThe translational momentum of the extended body $[ \mathrm{energy}^{1/2} \cdot \mathrm{mass}^{1/2}]$.
net_force: VThe net force applied to the body in a Microstate $[ \mathrm{energy}^{1/2} \cdot \mathrm{mass}^{1/2}]$.
net_virial: V::TensorThe net virial applied to the body in a Microstate $[\mathrm{energy}]$.
moment_of_inertia: R::MomentOfInertiaThe moment of inertia of the extended body $[\mathrm{mass} \cdot \mathrm{length}^2]$.
angular_momentum: R::AngularMomentumThe angular momentum of the extended body $[\mathrm{mass} \cdot \mathrm{length}^2]$.
net_torque: V::BivectorThe net torque applied to the body by others in a Microstate $[\mathrm{energy}]$.