pub struct OrientedHyperbolicPoint<const N: usize, R> {
pub position: Hyperbolic<N>,
pub orientation: R,
}Expand description
The position and orientation of an extended body in hyperbolic space.
Use OrientedHyperbolicPoint as a Body or
Site property type.
Example
use hoomd_manifold::{Hyperbolic, Minkowski};
use hoomd_microstate::property::OrientedHyperbolicPoint;
use hoomd_vector::Angle;
let point = OrientedHyperbolicPoint {
position: Hyperbolic::from_minkowski_coordinates(Minkowski::from([
0.0, 0.0, 1.0,
])),
orientation: Angle::from(2.39),
};Fields
position: Hyperbolic<N>The location of the extended body in the system frame.
orientation: ROrientation of the body in the global frame.
Implementations
Sourceimpl OrientedHyperbolicPoint<3, Angle>
impl OrientedHyperbolicPoint<3, Angle>
Sourcepub fn intersection_point(theta: f64, boost: f64) -> (f64, f64)
pub fn intersection_point(theta: f64, boost: f64) -> (f64, f64)
Compute the intersection of the geodesic arc passing through a point and the x axis, given the polar coordinates of the point. Returns a tuple containing the x coordinates of the intersection and the signed arc-angle of the path.
Sourcepub fn parallel_transport_angle(
start: &Hyperbolic<3>,
destination: &Hyperbolic<3>,
) -> f64
pub fn parallel_transport_angle( start: &Hyperbolic<3>, destination: &Hyperbolic<3>, ) -> f64
Compute the signed angle change when an oriented hyperbolic point is translated to another point.
Sourcepub fn deck_transform(
boost: f64,
rotation: f64,
position: &Hyperbolic<3>,
) -> f64
pub fn deck_transform( boost: f64, rotation: f64, position: &Hyperbolic<3>, ) -> f64
Compute the change in orientation associated with a transformation from a given
position. Isometries of hyperbolic space can be expressed as a boost conjugated
by a rotation, i.e., $R(\theta) T(\eta) R(-\theta)$. Such a transformation can
be expressed in the Poincaré disk representation as the Mobius transformation
g(z) = \begin{bmatrix} \cosh(\eta/2) & e^{i\theta} \sinh(\eta/2) \\ e^{-i\theta}\sinh(\eta/2) & \cosh(\eta/2) \end{bmatrix}This transformation induces a rotation of angle $\Delta\phi$ in the tangent
bundle, where
\Delta \phi = \operatorname{Arg} [g'(z)] = -2 \operatorname{Arg}[e^{-i\theta}\sinh(\eta/2)z + \cosh(\eta/2)]Sourcepub fn sample<R: Rng + ?Sized>(
disk: &HyperbolicDisk,
rng: &mut R,
) -> (Hyperbolic<3>, f64, f64)
pub fn sample<R: Rng + ?Sized>( disk: &HyperbolicDisk, rng: &mut R, ) -> (Hyperbolic<3>, f64, f64)
Generate a random HyperbolicDisk point with the corresponding boost rapidity
and rotation angle. Function returns a tuple with the random Hyperbolic<3>
point in the first entry, the associated rapidity and angle in the second and
third entries, respectively.
Panics
Panics when maximum boost is a non-positive number.