pub trait AngularMomentum {
type AngularMomentum;
// Required methods
fn angular_momentum(&self) -> &Self::AngularMomentum;
fn angular_momentum_mut(&mut self) -> &mut Self::AngularMomentum;
}Expand description
The rotational motion of a body: $\vec{L}$
AngularMomentum describes the rotational motion of the body in the body frame.
hoomd_md does not compute or utilize the angular momentum of sites.
Units
The units of AngularMomentum are $[\mathrm{mass}^{1/2} \cdot \mathrm{length} \cdot \mathrm{energy}^{1/2}]$.
Required Associated Types
Sourcetype AngularMomentum
type AngularMomentum
Type that can express the angular momentum of a site or body.
Required Methods
Sourcefn angular_momentum(&self) -> &Self::AngularMomentum
fn angular_momentum(&self) -> &Self::AngularMomentum
The angular momentum of this site or body $[\mathrm{mass}^{1/2} \cdot \mathrm{length} \cdot \mathrm{energy}^{1/2}]$.
Sourcefn angular_momentum_mut(&mut self) -> &mut Self::AngularMomentum
fn angular_momentum_mut(&mut self) -> &mut Self::AngularMomentum
The mutable angular momentum of this site or body $[\mathrm{mass}^{1/2} \cdot \mathrm{length} \cdot \mathrm{energy}^{1/2}]$.