pub trait Momentum {
type Momentum;
// Required methods
fn momentum(&self) -> &Self::Momentum;
fn momentum_mut(&mut self) -> &mut Self::Momentum;
fn velocity(&self) -> Self::Momentum;
fn set_velocity(&mut self, velocity: Self::Momentum);
}Expand description
The translational motion of a body: $\vec{p}$
Momentum describes the translational motion of the body relative to the origin of the
system coordinate system.
hoomd_md does not compute or utilize the momentum of sites.
Units
Momentum vectors have units of $[ \mathrm{energy}^{1/2} \cdot \mathrm{mass}^{1/2}]$.
Required Associated Types
Required Methods
Sourcefn momentum(&self) -> &Self::Momentum
fn momentum(&self) -> &Self::Momentum
The momentum of this body $[ \mathrm{energy}^{1/2} \cdot \mathrm{mass}^{1/2}]$.
Sourcefn momentum_mut(&mut self) -> &mut Self::Momentum
fn momentum_mut(&mut self) -> &mut Self::Momentum
The mutable momentum of this body $[ \mathrm{energy}^{1/2} \cdot \mathrm{mass}^{1/2}]$.
Sourcefn velocity(&self) -> Self::Momentum
fn velocity(&self) -> Self::Momentum
The velocity of this body $[ \mathrm{energy}^{1/2} \cdot \mathrm{mass}^{-1/2}]$.
Sourcefn set_velocity(&mut self, velocity: Self::Momentum)
fn set_velocity(&mut self, velocity: Self::Momentum)
Change the velocity of this body.