pub trait NetForce {
type NetForce;
// Required methods
fn net_force(&self) -> &Self::NetForce;
fn net_force_mut(&mut self) -> &mut Self::NetForce;
}Expand description
The total force acting on a site or body: $\vec{F}$
NetForce is set only for bodies that belong to a microstate. It is always in the
system frame.
hoomd_md does not store the net force acting on individual sites. Use methods in
hoomd_interaction to compute forces on sites when needed.
Units
Net force vectors have units of $[\mathrm{energy} \cdot \mathrm{length}^{-1}]$.
Required Associated Types
Required Methods
Sourcefn net_force(&self) -> &Self::NetForce
fn net_force(&self) -> &Self::NetForce
The net force on this body $[\mathrm{energy} \cdot \mathrm{length}^{-1}]$.
Sourcefn net_force_mut(&mut self) -> &mut Self::NetForce
fn net_force_mut(&mut self) -> &mut Self::NetForce
The mutable net force on this body $[\mathrm{energy} \cdot \mathrm{length}^{-1}]$.