pub trait NetVirial {
type NetVirial;
// Required methods
fn net_virial(&self) -> &Self::NetVirial;
fn net_virial_mut(&mut self) -> &mut Self::NetVirial;
}Expand description
The total virial acting on a site or body: $\mathbf{W}$
NetVirial is set only for bodies that belong to a microstate. It is always in the
system frame.
hoomd_md does not store the net virial acting on individual sites. Use methods in
hoomd_interaction to compute virials on sites when needed.
Units
Net virial matrices have units of $[\mathrm{energy}$.
Required Associated Types
Required Methods
Sourcefn net_virial(&self) -> &Self::NetVirial
fn net_virial(&self) -> &Self::NetVirial
The net virial on this body $[\mathrm{energy}]$.
Sourcefn net_virial_mut(&mut self) -> &mut Self::NetVirial
fn net_virial_mut(&mut self) -> &mut Self::NetVirial
The mutable net virial on this body $[\mathrm{energy}]$.