pub trait RotationalMotionTypes {
type MomentOfInertia;
type AngularMomentum;
}Expand description
Moment of inertia and angular momentum types.
RotationalMotionTypes sets which structs store the moment of inertia
and angular momentum for a given Rotation representation.
Required Associated Types
Sourcetype MomentOfInertia
type MomentOfInertia
Type that stores the moment of inertia in the natural coordinate frame of the body’s local rotation.
Sourcetype AngularMomentum
type AngularMomentum
Type that stores the angular momentum.